import pytest

from nasap_net.classes.validations import validate_name_of_component_kind


def test_validate_name_of_component_kind():
    pass


if __name__ == '__main__':
    pytest.main(['-vv', __file__])
